/*
 * statemachine.cpp
 *
 *  Created on: 21 Sep 2023
 *      Author: Dario
 */

#include <sta/tacos/statemachine.hpp>

#include <sta/debug/debug.hpp>


namespace sta
{
	namespace tacos
	{
		Statemachine::Statemachine()
			: TacosThread{"Statemachine", STA_TACOS_STATEMACHINE_PRIORITY},
			  currentState_{STA_TACOS_INITIAL_STATE},
			  lockoutTimer_{[](void *){}, nullptr},
			  failsafeTimer_{[](void *){}, nullptr},
			  transitionFunc_{[](uint16_t) -> uint16_t { return Statemachine::instance()->getCurrentState(); }}
		{
			STA_ASSERT(STA_TACOS_INITIAL_STATE < STA_TACOS_NUM_STATES);
		}

		void Statemachine::init()
		{
			lockoutTimer_.setCallback([](void *) {  }, nullptr);
			lockoutTimer_.start(5000);

			failsafeTimer_.setCallback([](void *) {
				Statemachine::instance()->forceStateTransition(1);
			}, nullptr);
			failsafeTimer_.start(10000);
		}

		void Statemachine::func()
		{
			uint16_t next = transitionFunc_(currentState_);

			STA_ASSERT(next < STA_TACOS_NUM_STATES);

			if (next != currentState_)
			{
				currentState_ = next;

				Statemachine::stateChangeEvent->set(TACOS_STATE_TRANSITION_FLAG);
			}

			Statemachine::stateChangeEvent->clear(TACOS_STATE_CHANGE_ALL_FLAG);
		}

		uint16_t Statemachine::getCurrentState() const
		{
			return currentState_;
		}

		void Statemachine::setStateTransitionFunction(std::function<uint16_t(uint16_t)> function)
		{
			STA_ASSERT(function != nullptr);

			transitionFunc_ = function;
		}

		void Statemachine::forceStateTransition(uint16_t state)
		{
			currentState_ = state;

			Statemachine::stateChangeEvent->set(TACOS_STATE_CHANGE_FORCED_FLAG);
			Statemachine::stateChangeEvent->clear(TACOS_STATE_CHANGE_ALL_FLAG);
		}

		Statemachine* Statemachine::_instance = nullptr;
		RtosEvent* Statemachine::stateChangeEvent = nullptr;

	}  // namespace tacos
}  // namespace sta



